subroutine tight_bind
  use commons
!c
!c----------------------------------------------------------------------------
!c
!c                  tight binding electronic structure
!c
!c       original version by k. laasonen and r. virkkunen
!c
!c
!c       the program reads the atomic coordinates and calculates the
!c       tight-binding matrix h, the core-core energy, and the total
!c       energy.
!c
!c       the eigenvalues e are calculated from equation  h x = e x
!c       (where h is hermitian matrix) using attached eispack routines.
!c
!c       units:   ev = 1    a = 1
!c
!c       only the gamma point in used in this version
!c
!c
!c----------------------------------------------------------------------------
!c
!c
  implicit double precision(a-h,o-z)
!      include 'common_files.inc'
  external e_corr
!c
!c      naapurit = neighbours, paikat = places (coordinates) of the atoms,
!c

!c----------------------------------------------------------------------------
!c
!c
!c         tb parametrization for the hamiltonian is from
!c         xu, wang, chan, and ho, j. phys.: condens. matter 4 (1992) 6047.
!c

!c      neigen=number of occupied eigenstates
!c      emintb,emax: minimum and maximum eigenenergies
!c      basis=basis vectors of the lattice
!c      rcut=distance determining if a neighbor atom is a nearest neighbor or notc
!c
      if(np.ne.natx) then
            write(*,*) 'set natx=',np,' in common_tb.inc and recompile'
!            include 'close.inc'
            stop
      endif
      do i=1,3
        do j=1,3
           basis(i,j) = 0.0d0
        enddo
      enddo
      do i=1,3
           basis(i,i) = cube(i)
      enddo

       rcut = 2.60d0
!c      rcut = 2.0d0
       neigen=2*np
!c
!c      k-points (only the gamma-point is used in this version)
!c       read(8,*) nk
!c       do 40 i=1,nk
!c         read(8,*) (xkt(i,j),j=1,3)
!c       do 40 j=1,3
!c         xkt(i,j) = 2*pi/scale*xkt(i,j)
!c40     continue

!c----------------------------------------------------------------------------

!c energy for separated atoms

!c classical contribution:
       e_cl = np*f0
!c quantum contribution:
       e_qu = np*(2.0d0*tbpar(1) + 2.0d0*tbpar(2))
       e_separ = e_cl + e_qu
       ip = 1
       n  = 4*np
       iik = 0
       ne_yli = 0

!c      determines the nearest neighbors
       call neighbor

!c      calculates hamiltonian matrix
       call laske_h


       ifail = 1
!c eispack call
       ierr = 0
       matz = 1


!c real hermitian matrix
       call rs(nmax,nmax,hr,e,matz,vr,fv1,fv2,ierr)

!c
!c write out all eigenvectors 
!c
!c      do i=1,256
!c           do j=1,256
!c                write(36,900) i,j,vr(i,j)
!c           enddo
!c      enddo
!c900   format(2i6,f12.6)

!c
!c calculate pair energy 
       ecl = e_corr(i)

!c calculate forces
      call force

!c
!c      calculate the potential energy
!c
       summa = 0.0d0
       do i = 1,neigen
           summa = summa + 2.*e(i)
       enddo

       tote = summa + ecl  - e_separ
       write(*,*) 'band energy,bond energy= ',summa,summa - e_qu

!c  write final global and local densities of states, eigenenergies 
!c and eigenvectors 
      if(lstep.eq.kvc) call dos

      return

94     format('  bs energy ',f13.6,'  core energy ',f13.6, /, &
            'total pe/atom ',f13.6)
       end

