!
!     Shell model FORCE & ENERGY CALCULATION for PbTiO3
!     
!     to be published?
!
!     Last update 19/09/2006
!

      subroutine shellmodel_PTO
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision alarge(4),rho(4),clarge(4),rcsr(4),rtp(4)
      double precision q(3,2),springc(3),springc2(3)
      double precision :: rec(3,3) 
      double precision :: ewaldgm,acc,deth,efpe
      double precision :: errfc
      integer :: istat
      integer :: ksp
      integer :: kint(3,3)



!-----Shell model parameters for PbTiO3
      ! Pb-1, O-2, Ti-3  (ksp) !
      
      ! Interaction number
      kint(1:3,1:3)=0  ! Reset
      kint(1,2)=1  ! Pb-O
      kint(2,1)=1  ! Pb-O
      kint(2,2)=2  ! O -O
      kint(3,2)=3  ! Ti-O
      kint(2,3)=3  ! Ti-O
      kint(1,3)=4  ! Pb-Ti
      kint(3,1)=4  ! Pb-Ti
      
      ! Charge (e)
      q(1,1)=5.495849d0         ! Pb Core
      q(1,2)=-3.632074d0        ! Pb Shell
      q(2,1)=2.540879d0         ! O  Core
      q(2,2)=-4.192555d0        ! O  Shell
      q(3,1)=19.369090d0        ! Ti Core
      q(3,2)=-16.277840d0       ! Ti Shell 

      ! Spring constant k2, k4 (eV/Angst**2,eV/Angst**4)
      springc(1)=154.87128d0    ! Pb
      springc(2)=180.91341d0    ! O
      springc(3)=8813.0140d0    ! Ti
      springc2(1)=21822.057d0   ! Pb
      springc2(2)=6945.7814d0   ! O
      springc2(3)=1926889.6d0   ! Ti

      ! Short-range potential parameters
      ! Aij (eV)
      alarge(1)=2551.7924d0     ! Pb-O
      alarge(2)=1698.6653d0     ! O -O
      alarge(3)=2555.2075d0     ! Ti-O
      alarge(4)=397.63568d0     ! Pb-Ti
      ! rhoij (Angst)
      rho(1)=0.300415d0         ! Pb-O
      rho(2)=0.271756d0         ! O -O
      rho(3)=0.278391d0         ! Ti-O
      rho(4)=0.394957d0         ! Pb-Ti
      ! Cij (eV*Angst**6)
      clarge(1)=2.6167608d0     ! Pb-O
      clarge(2)=60.861640d0     ! O -O
      clarge(3)=2.6009265d0     ! Ti-O
      clarge(4)=231.06642d0     ! Pb-Ti
      ! Cutoff radius (Angst)
      rcsr(1)=8.0d0             ! Pb-O
      rcsr(2)=8.0d0             ! O -O
      rcsr(3)=8.0d0             ! Ti-O
      rcsr(4)=8.0d0             ! Pb-Ti
      ! Taper range (Angst)
      rtp(1)=2.0d0              ! Pb-O
      rtp(2)=2.0d0              ! O -O
      rtp(3)=2.0d0              ! Ti-O
      rtp(4)=2.0d0              ! Pb-Ti
!-----------------------------------------------------------------------




!-----Parameters for Ewald method  -Molecular Simulation, vol.1, pp.207-224 (1988)-
!     e**2/4*pi*epsil0
      efpe=elc**2/(4.0d0*pi*epsil0)
!     Ewald parameter (1/m)
      ewaldgm=(dble(n)*pi**3.0d0/v**2.0d0)**(1.0d0/6.0d0)
!     Ewald 1st term
      naccew=12
      acc=(1.0d-1)**dble(naccew)
      rce1=dsqrt(-dlog(acc))/ewaldgm ! (m)
!     Ewald 2nd term
      deth= hmat(1,1)*hmat(2,2)*hmat(3,3) + hmat(1,2)*hmat(2,3)*hmat(3,1) &
      +     hmat(1,3)*hmat(2,1)*hmat(3,2) - hmat(1,1)*hmat(2,3)*hmat(3,2) &
      -     hmat(1,2)*hmat(2,1)*hmat(3,3) - hmat(1,3)*hmat(2,2)*hmat(3,1)
      call recipro(hmat,rec)
      rec1=2.0d0*pi*dsqrt(rec(1,1)**2+rec(2,1)**2+rec(3,1)**2)/deth
      rec2=2.0d0*pi*dsqrt(rec(1,2)**2+rec(2,2)**2+rec(3,2)**2)/deth
      rec3=2.0d0*pi*dsqrt(rec(1,3)**2+rec(2,3)**2+rec(3,3)**2)/deth
      gce2=2.0d0*ewaldgm*dsqrt(-dlog(acc))
      mewald2x=int(gce2/rec1)+1
      mewald2y=int(gce2/rec2)+1
      mewald2z=int(gce2/rec3)+1


!---Cutoff radius
      rc=0.0d0
      rc=max(rce1,rc)
      do i=1,4
         rc=max(rcsr(i)*1.0d-10,rc)
      enddo
      rc2=rc**2


!--
      istat=1

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0

!   Reset energy and force
      enpot=0.0d0
      pewald1=0.0d0
      pewald2=0.0d0
      pewald3=0.0d0
      pewald4=0.0d0
      psr=0.0d0
      pharmonic=0.0d0
      tmpfx(1:n/2,1:2)=0.0d0
      tmpfy(1:n/2,1:2)=0.0d0
      tmpfz(1:n/2,1:2)=0.0d0


!---Copy rxyz(n) to tmprxyz(n/2,2)
      do i=1,n/2
         tmprx(i,1)=rx(i)
         tmpry(i,1)=ry(i)
         tmprz(i,1)=rz(i)
         tmprx(i,2)=rx(i+n/2)
         tmpry(i,2)=ry(i+n/2)
         tmprz(i,2)=rz(i+n/2)
      enddo

      
!-----Ewald 1st term and Short-range interaction
      kns=0
      do 1000 i=1,n/2
         kn=knum(i)
         if (kn.eq.0) go to 1000
         do 1100 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)

            !!! Questionable... check bookkeep algorithm
            if ((j.gt.n/2).or.(j.lt.i)) goto 1100

            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            do 1200 k=1,2    ! Core,Shell loop
               do 1300 l=1,2 ! Core,Shell loop
                  xx=tmprx(i,k)-tmprx(j,l)
                  yy=tmpry(i,k)-tmpry(j,l)
                  zz=tmprz(i,k)-tmprz(j,l)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  if (istat == 1) then
                     r2=xx**2+yy**2+zz**2
                     r=sqrt(r2)
                     rr=r*1.0d10

                     ! Ewald 1st term (Real space)
                     if (r.gt.rce1) goto 1400
                     if ((i.eq.j).and.(rr.lt.1.0d0)) goto 1400
                     ! Energy (J)
                     pewaldeach1=q(ksp(asp(i)),k)*q(ksp(asp(j)),l)*efpe*errfc(ewaldgm*r)/r
                     ! Force(N)
                     f=q(ksp(asp(i)),k)*q(ksp(asp(j)),l)*efpe*(errfc(ewaldgm*r)/r**2 &
                          +2.0d0*ewaldgm/dsqrt(pi)*dexp(-(ewaldgm*r)**2)/r)
                     fxx=f/r*xx
                     fyy=f/r*yy
                     fzz=f/r*zz

                     if (i.eq.j) then
                        tmpfx(i,k)=tmpfx(i,k)+fxx
                        tmpfy(i,k)=tmpfy(i,k)+fyy
                        tmpfz(i,k)=tmpfz(i,k)+fzz                        
                        dmat(1,1)=dmat(1,1)+f/r*xx*xx/2.0d0
                        dmat(1,2)=dmat(1,2)+f/r*xx*yy/2.0d0
                        dmat(1,3)=dmat(1,3)+f/r*xx*zz/2.0d0
                        dmat(2,1)=dmat(1,2)
                        dmat(2,2)=dmat(2,2)+f/r*yy*yy/2.0d0
                        dmat(2,3)=dmat(2,3)+f/r*yy*zz/2.0d0
                        dmat(3,1)=dmat(1,3)
                        dmat(3,2)=dmat(2,3)
                        dmat(3,3)=dmat(3,3)+f/r*zz*zz/2.0d0                        
                        pewald1=pewald1+pewaldeach1/2.0d0
                     else
                        tmpfx(i,k)=tmpfx(i,k)+fxx
                        tmpfy(i,k)=tmpfy(i,k)+fyy
                        tmpfz(i,k)=tmpfz(i,k)+fzz
                        tmpfx(j,l)=tmpfx(j,l)-fxx
                        tmpfy(j,l)=tmpfy(j,l)-fyy
                        tmpfz(j,l)=tmpfz(j,l)-fzz                        
                        dmat(1,1)=dmat(1,1)+f/r*xx*xx
                        dmat(1,2)=dmat(1,2)+f/r*xx*yy
                        dmat(1,3)=dmat(1,3)+f/r*xx*zz
                        dmat(2,1)=dmat(1,2)
                        dmat(2,2)=dmat(2,2)+f/r*yy*yy
                        dmat(2,3)=dmat(2,3)+f/r*yy*zz
                        dmat(3,1)=dmat(1,3)
                        dmat(3,2)=dmat(2,3)
                        dmat(3,3)=dmat(3,3)+f/r*zz*zz                        
                        pewald1=pewald1+pewaldeach1
                     endif
1400                 continue
                     
                     
                     ! Short-range (Buckingham*taper)
                     if ((kint(ksp(asp(i)),ksp(asp(j))).eq.0).or.(k.ne.2).or.(l.ne.2)) goto 1500
                     kinter=kint(ksp(asp(i)),ksp(asp(j)))
                     if (rr.gt.rcsr(kinter)) goto 1500
                     
                     f=(alarge(kinter)/rho(kinter)*dexp(-rr/rho(kinter))-6.0d0*clarge(kinter)/rr**7)*ev*1.0d10
                     psreach=(alarge(kinter)*dexp(-rr/rho(kinter))-clarge(kinter)/rr**6)*ev
                     ! Tapering (rt < rr < rc)
                     if ((rr.gt.rcsr(kinter)-rtp(kinter)).and.(rr.le.rcsr(kinter))) then
                        psreach=psreach*0.50d0 &
                             *(1.0d0+dcos(pi*(rr-rcsr(kinter)-rtp(kinter))/(rcsr(kinter)-rcsr(kinter)-rtp(kinter))))
                        f=((alarge(kinter)/rho(kinter)*dexp(-rr/rho(kinter))-6.0d0*clarge(kinter)/rr**7) &
                             *0.50d0*(1.0d0+dcos(pi*(rr-(rcsr(kinter)-rtp(kinter)))/rtp(kinter))) &
                             -(alarge(kinter)*dexp(-rr/rho(kinter))-clarge(kinter)/rr**6) &
                             *(-0.50d0*pi/rtp(kinter)*dsin(pi*(rr-(rcsr(kinter)-rtp(kinter)))/rtp(kinter))))*ev*1.0d10
                     endif
                     fxx=f/r*xx
                     fyy=f/r*yy
                     fzz=f/r*zz

                     if (i.eq.j) then
                        tmpfx(i,2)=tmpfx(i,2)+fxx
                        tmpfy(i,2)=tmpfy(i,2)+fyy
                        tmpfz(i,2)=tmpfz(i,2)+fzz
                        dmat(1,1)=dmat(1,1)+f/r*xx*xx/2.0d0
                        dmat(1,2)=dmat(1,2)+f/r*xx*yy/2.0d0
                        dmat(1,3)=dmat(1,3)+f/r*xx*zz/2.0d0
                        dmat(2,1)=dmat(1,2)
                        dmat(2,2)=dmat(2,2)+f/r*yy*yy/2.0d0
                        dmat(2,3)=dmat(2,3)+f/r*yy*zz/2.0d0
                        dmat(3,1)=dmat(1,3)
                        dmat(3,2)=dmat(2,3)
                        dmat(3,3)=dmat(3,3)+f/r*zz*zz/2.0d0
                        psr=psr+psreach/2.0d0
                     else
                        tmpfx(i,2)=tmpfx(i,2)+fxx
                        tmpfy(i,2)=tmpfy(i,2)+fyy
                        tmpfz(i,2)=tmpfz(i,2)+fzz
                        tmpfx(j,2)=tmpfx(j,2)-fxx
                        tmpfy(j,2)=tmpfy(j,2)-fyy
                        tmpfz(j,2)=tmpfz(j,2)-fzz
                        dmat(1,1)=dmat(1,1)+f/r*xx*xx
                        dmat(1,2)=dmat(1,2)+f/r*xx*yy
                        dmat(1,3)=dmat(1,3)+f/r*xx*zz
                        dmat(2,1)=dmat(1,2)
                        dmat(2,2)=dmat(2,2)+f/r*yy*yy
                        dmat(2,3)=dmat(2,3)+f/r*yy*zz
                        dmat(3,1)=dmat(1,3)
                        dmat(3,2)=dmat(2,3)
                        dmat(3,3)=dmat(3,3)+f/r*zz*zz
                        psr=psr+psreach
                     endif
 1500                continue

                  endif
 1300          continue 
 1200       continue
 1100    continue
 1000 continue

               
               
!-----Ewald 2nd term (Reciprocal space)
      do 2000 mx=-mewald2x,mewald2x
      do 2000 my=-mewald2y,mewald2y
      do 2000 mz=-mewald2z,mewald2z
         
         if ((mx.eq.0).and.(my.eq.0).and.(mz.eq.0)) goto 2000
         gbx=2.0d0*pi/v*(rec(1,1)*dble(mx)+rec(1,2)*dble(my)+rec(1,3)*dble(mz))
         gby=2.0d0*pi/v*(rec(2,1)*dble(mx)+rec(2,2)*dble(my)+rec(2,3)*dble(mz))
         gbz=2.0d0*pi/v*(rec(3,1)*dble(mx)+rec(3,2)*dble(my)+rec(3,3)*dble(mz))
         gbabs2=gbx**2+gby**2+gbz**2
         gbabs=dsqrt(gbabs2)
         if (gbabs.gt.gce2) goto 2000 
         
         ! Make cossum and sinsum
         cossum=0.0d0
         sinsum=0.0d0
         do i=1,n/2
            do j=1,2
               gbr=tmprx(i,j)*gbx+tmpry(i,j)*gby+tmprz(i,j)*gbz 
               cossum=cossum+q(ksp(asp(i)),j)*dcos(gbr)
               sinsum=sinsum+q(ksp(asp(i)),j)*dsin(gbr)
            enddo
         enddo
         
         ! Force (N) 
         do i=1,n/2
            do j=1,2
               gbr=tmprx(i,j)*gbx+tmpry(i,j)*gby+tmprz(i,j)*gbz
               f=4.0d0*pi/v/gbabs*dexp(-gbabs2/4.0d0/ewaldgm**2) &
                    *q(ksp(asp(i)),j)*(dsin(gbr)*cossum-dcos(gbr)*sinsum)*efpe
               fxx=f/gbabs*gbx
               fyy=f/gbabs*gby
               fzz=f/gbabs*gbz
               tmpfx(i,j)=tmpfx(i,j)+fxx
               tmpfy(i,j)=tmpfy(i,j)+fyy
               tmpfz(i,j)=tmpfz(i,j)+fzz
            enddo
         enddo
         
         ! Stress
         ew2a=2.0d0*pi/v*efpe*dexp(-gbabs2/4.0d0/ewaldgm**2)/gbabs2*(cossum**2+sinsum**2)   
         ew2b=2.0d0/gbabs2*(gbabs2/4.0d0/ewaldgm**2+1.0d0)
         dmat(1,1)=dmat(1,1)-ew2a*(ew2b*gbx*gbx-1.0d0)
         dmat(1,2)=dmat(1,2)-ew2a*ew2b*gbx*gby
         dmat(1,3)=dmat(1,3)-ew2a*ew2b*gbx*gbz
         dmat(2,1)=dmat(1,2)
         dmat(2,2)=dmat(2,2)-ew2a*(ew2b*gby*gby-1.0d0)
         dmat(2,3)=dmat(2,3)-ew2a*ew2b*gby*gbz
         dmat(3,1)=dmat(1,3)
         dmat(3,2)=dmat(2,3)
         dmat(3,3)=dmat(3,3)-ew2a*(ew2b*gbz*gbz-1.0d0)
         
         ! Energy (J)
         pewaldeach2=2.0d0*pi/v*efpe*dexp(-gbabs2/4.0d0/ewaldgm**2)/gbabs2*(cossum**2+sinsum**2)
         pewald2=pewald2+pewaldeach2

 2000 continue


  
!-----Ewald 3rd term
      do 3000 i=1,n/2
         do 3100 j=1,2
            pewald3=pewald3-q(ksp(asp(i)),j)**2*ewaldgm/dsqrt(pi)*efpe
 3100    continue
 3000 continue



!-----Ewald 4th term(Correction) and Core-Shell interaction(Spring)
      do 4000 i=1,n/2
         xx=tmprx(i,1)-tmprx(i,2)
         yy=tmpry(i,1)-tmpry(i,2)
         zz=tmprz(i,1)-tmprz(i,2)
         r2=xx**2+yy**2+zz**2
         r=dsqrt(r2)
         
         ! Ewald 4th term
         ! Energy (J)
         if (r.lt.1.0d-11) then
            pewaldeach4=-q(ksp(asp(i)),1)*q(ksp(asp(i)),2)*efpe*2.0d0/dsqrt(pi) &
                 *(ewaldgm-r**2*ewaldgm**3/3.0d0+r**4*ewaldgm**5/10.0d0 &
                 -r**6*ewaldgm**7/42.0d0)
         else
            pewaldeach4=-q(ksp(asp(i)),1)*q(ksp(asp(i)),2)*efpe*(1.0d0-errfc(ewaldgm*r))/r
         endif
         pewald4=pewald4+pewaldeach4
         
         ! Force (N)
         if (r.eq.0.0d0) then
            goto 4100
         elseif (r.lt.1.0d-11) then
            f=q(ksp(asp(i)),1)*q(ksp(asp(i)),2)*efpe*2.0d0/dsqrt(pi)*(-2.0d0/3.0d0*ewaldgm**3*r &
                 +2.0d0/5.0d0*ewaldgm**5*r**3-1.0d0/7.0d0*ewaldgm**7*r**5 &
                 +1.0d0/27.0d0*ewaldgm**9*r**7)
         else
            f=q(ksp(asp(i)),1)*q(ksp(asp(i)),2)*efpe*(2.0d0*ewaldgm/dsqrt(pi) &
                 *dexp(-(ewaldgm*r)**2)/r-(1.0d0-errfc(ewaldgm*r))/r**2)
         endif
         fxx=f/r*xx
         fyy=f/r*yy
         fzz=f/r*zz

         tmpfx(i,1)=tmpfx(i,1)+fxx
         tmpfy(i,1)=tmpfy(i,1)+fyy
         tmpfz(i,1)=tmpfz(i,1)+fzz
         tmpfx(i,2)=tmpfx(i,2)-fxx
         tmpfy(i,2)=tmpfy(i,2)-fyy
         tmpfz(i,2)=tmpfz(i,2)-fzz
         dmat(1,1)=dmat(1,1)+f/r*xx*xx
         dmat(1,2)=dmat(1,2)+f/r*xx*yy
         dmat(1,3)=dmat(1,3)+f/r*xx*zz
         dmat(2,1)=dmat(1,2)
         dmat(2,2)=dmat(2,2)+f/r*yy*yy
         dmat(2,3)=dmat(2,3)+f/r*yy*zz
         dmat(3,1)=dmat(1,3)
         dmat(3,2)=dmat(2,3)
         dmat(3,3)=dmat(3,3)+f/r*zz*zz


         ! Core-Shell interaction(Spring)
         if (r.eq.0.0d0) goto 4100
         pheach=(0.5d0*springc(ksp(asp(i)))*(r2*1.0d20)+1.0d0/24.0d0*springc2(ksp(asp(i)))*(r2**2*1.0d40))*ev
         pharmonic=pharmonic+pheach

         f=-(springc(ksp(asp(i)))*(r*1.0d10)+1.0d0/6.0d0*springc2(ksp(asp(i)))*(r**3*1.0d30))*ev*1.0d10
         fxx=f/r*xx
         fyy=f/r*yy
         fzz=f/r*zz

         tmpfx(i,1)=tmpfx(i,1)+fxx
         tmpfy(i,1)=tmpfy(i,1)+fyy
         tmpfz(i,1)=tmpfz(i,1)+fzz
         tmpfx(i,2)=tmpfx(i,2)-fxx
         tmpfy(i,2)=tmpfy(i,2)-fyy
         tmpfz(i,2)=tmpfz(i,2)-fzz
         dmat(1,1)=dmat(1,1)+f/r*xx*xx
         dmat(1,2)=dmat(1,2)+f/r*xx*yy
         dmat(1,3)=dmat(1,3)+f/r*xx*zz
         dmat(2,1)=dmat(1,2)
         dmat(2,2)=dmat(2,2)+f/r*yy*yy
         dmat(2,3)=dmat(2,3)+f/r*yy*zz
         dmat(3,1)=dmat(1,3)
         dmat(3,2)=dmat(2,3)
         dmat(3,3)=dmat(3,3)+f/r*zz*zz
 4100    continue
 4000 continue


!         print *,pewald1/ev,pewald2/ev,pewald3/ev,pewald4/ev,psr/ev,pharmonic/ev
!         print *,(pewald1+pewald2+pewald3+pewald4+psr+pharmonic)/ev


!---Etot---
      enpot=pewald1+pewald2+pewald3+pewald4+psr+pharmonic
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)
      do i=1,n/2
         fx(i)=tmpfx(i,1)
         fy(i)=tmpfy(i,1)
         fz(i)=tmpfz(i,1)
         fx(i+n/2)=tmpfx(i,2)
         fy(i+n/2)=tmpfy(i,2)
         fz(i+n/2)=tmpfz(i,2)
      enddo

      end






!-----Species number ksp
      integer function ksp(species)
        character(2) :: species
        if (species=='Pb') ksp=1
        if (species=='O ') ksp=2
        if (species=='Ti') ksp=3
      end function ksp


!-----Complementary error function
      double precision function errfc(x)
      implicit real*8 (a - h, o - z)
      dimension a(0 : 64), b(0 : 64)
      data (a(i), i = 0, 12) / &
         0.00000000005958930743d0, -0.00000000113739022964d0, &
         0.00000001466005199839d0, -0.00000016350354461960d0, &
         0.00000164610044809620d0, -0.00001492559551950604d0, &
         0.00012055331122299265d0, -0.00085483269811296660d0, &
         0.00522397762482322257d0, -0.02686617064507733420d0, &
         0.11283791670954881569d0, -0.37612638903183748117d0, &
         1.12837916709551257377d0 / 
      data (a(i), i = 13, 25) / &
         0.00000000002372510631d0, -0.00000000045493253732d0, &
         0.00000000590362766598d0, -0.00000006642090827576d0, &
         0.00000067595634268133d0, -0.00000621188515924000d0, &
         0.00005103883009709690d0, -0.00037015410692956173d0, &
         0.00233307631218880978d0, -0.01254988477182192210d0, &
         0.05657061146827041994d0, -0.21379664776456006580d0, &
         0.84270079294971486929d0 / 
      data (a(i), i = 26, 38) / &
         0.00000000000949905026d0, -0.00000000018310229805d0, &
         0.00000000239463074000d0, -0.00000002721444369609d0, &
         0.00000028045522331686d0, -0.00000261830022482897d0, &
         0.00002195455056768781d0, -0.00016358986921372656d0, &
         0.00107052153564110318d0, -0.00608284718113590151d0, &
         0.02986978465246258244d0, -0.13055593046562267625d0, &
         0.67493323603965504676d0 / 
      data (a(i), i = 39, 51) / &
         0.00000000000382722073d0, -0.00000000007421598602d0, &
         0.00000000097930574080d0, -0.00000001126008898854d0, &
         0.00000011775134830784d0, -0.00000111992758382650d0, &
         0.00000962023443095201d0, -0.00007404402135070773d0, &
         0.00050689993654144881d0, -0.00307553051439272889d0, &
         0.01668977892553165586d0, -0.08548534594781312114d0, &
         0.56909076642393639985d0 / 
      data (a(i), i = 52, 64) / &
         0.00000000000155296588d0, -0.00000000003032205868d0, &
         0.00000000040424830707d0, -0.00000000471135111493d0, &
         0.00000005011915876293d0, -0.00000048722516178974d0, &
         0.00000430683284629395d0, -0.00003445026145385764d0, &
         0.00024879276133931664d0, -0.00162940941748079288d0, &
         0.00988786373932350462d0, -0.05962426839442303805d0, &
         0.49766113250947636708d0 / 
      data (b(i), i = 0, 12) / &
         -0.00000000029734388465d0, 0.00000000269776334046d0, &
         -0.00000000640788827665d0, -0.00000001667820132100d0, &
         -0.00000021854388148686d0, 0.00000266246030457984d0, &
          0.00001612722157047886d0, -0.00025616361025506629d0, &
          0.00015380842432375365d0, 0.00815533022524927908d0, &
         -0.01402283663896319337d0, -0.19746892495383021487d0, &
         0.71511720328842845913d0 / 
      data (b(i), i = 13, 25) / &
         -0.00000000001951073787d0, -0.00000000032302692214d0, & 
         0.00000000522461866919d0, 0.00000000342940918551d0, &
         -0.00000035772874310272d0, 0.00000019999935792654d0, &
         0.00002687044575042908d0, -0.00011843240273775776d0, &
         -0.00080991728956032271d0, 0.00661062970502241174d0, &
         0.00909530922354827295d0, -0.20160072778491013140d0, &
         0.51169696718727644908d0 / 
      data (b(i), i = 26, 38) / &
         0.00000000003147682272d0, -0.00000000048465972408d0, &
         0.00000000063675740242d0, 0.00000003377623323271d0, &
         -0.00000015451139637086d0, -0.00000203340624738438d0, & 
         0.00001947204525295057d0, 0.00002854147231653228d0, &
         -0.00101565063152200272d0, 0.00271187003520095655d0, &
         0.02328095035422810727d0, -0.16725021123116877197d0, &
         0.32490054966649436974d0 / 
      data (b(i), i = 39, 51) / &
         0.00000000002319363370d0, -0.00000000006303206648d0, &
         -0.00000000264888267434d0, 0.00000002050708040581d0, &
         0.00000011371857327578d0, -0.00000211211337219663d0, &
         0.00000368797328322935d0, 0.00009823686253424796d0, &
         -0.00065860243990455368d0, -0.00075285814895230877d0, &
         0.02585434424202960464d0, -0.11637092784486193258d0, &
         0.18267336775296612024d0 / 
      data (b(i), i = 52, 64) / &
         -0.00000000000367789363d0, 0.00000000020876046746d0, &
         -0.00000000193319027226d0, -0.00000000435953392472d0, &
         0.00000018006992266137d0, -0.00000078441223763969d0, &
         -0.00000675407647949153d0, 0.00008428418334440096d0, &
         -0.00017604388937031815d0, -0.00239729611435071610d0, &
         0.02064129023876022970d0, -0.06905562880005864105d0, &
         0.09084526782065478489d0 / 
      w = abs(x)
      if (w .lt. 2.2d0) then
          t = w * w
          k = int(t)
          t = t - k
          k = k * 13
          y = ((((((((((((a(k) * t + a(k + 1)) * t + &
             a(k + 2)) * t + a(k + 3)) * t + a(k + 4)) * t + &
             a(k + 5)) * t + a(k + 6)) * t + a(k + 7)) * t + &
             a(k + 8)) * t + a(k + 9)) * t + a(k + 10)) * t + &
             a(k + 11)) * t + a(k + 12)) * w
      else if (w .lt. 6.9d0) then
          k = int(w)
          t = w - k
          k = 13 * (k - 2)
          y = (((((((((((b(k) * t + b(k + 1)) * t + &
             b(k + 2)) * t + b(k + 3)) * t + b(k + 4)) * t + &
             b(k + 5)) * t + b(k + 6)) * t + b(k + 7)) * t + &
             b(k + 8)) * t + b(k + 9)) * t + b(k + 10)) * t + &
             b(k + 11)) * t + b(k + 12)
          y = y * y
          y = y * y
          y = y * y
          y = 1 - y * y
      else
          y = 1
      end if
      if (x .lt. 0) y = -y
      errfc = 1.0d0-y
      end
