!
! Subroutine for voronoi polyhedron analysis
!
      subroutine voronoi
      use variables
      use constants
      use bookkeep_param
      implicit double precision (a-h,o-z)
      double precision, allocatable :: xact(:),yact(:),zact(:),vec(:)
      integer, allocatable :: no(:),np(:,:),na(:)
      double precision, allocatable :: rp(:,:)
      character(9) :: filename
      logical :: out
      integer :: nvor_count(30)

! for output
      filename='voronoi.d'
      ifile=7
      open (ifile,FILE=filename)

      allocate (xact(n),yact(n),zact(n),vec(n))
      allocate (no(n),np(n,3),rp(n,3),na(n))
      vec=0.0d0

      rc_vor2 = rc_vor**2
      rc_store=frc
      frc=rc_vor
      call bookkeep
      frc=rc_store

      nvor_count=0
      do i=1,n                  ! LOOP
!  search and remember neighboring atoms
         nc=0
         kns0=1
         if (i>1) then
            do jj=1,i-1
               kns0=kns0+knum(jj)
            enddo
         endif
         if (knum(i) /= 0) then
            do kns=kns0,kns0+knum(i)-1
               j=idpt(kns,1)
               ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
               xx=rx(i)-rx(j)
               yy=ry(i)-ry(j)
               zz=rz(i)-rz(j)
               call calc_replica(xx,yy,zz,rc_vor,ix,iy,iz,istat)
               if (istat == 1) then
                  r2=xx**2+yy**2+zz**2
                  if (r2.lt.rc_vor2) then
                     nc=nc+1
                     xact(nc)=xx; yact(nc)=yy; zact(nc)=zz
                  endif
               endif
            enddo
         endif
!   
         if (i>1) then
            kns=0
            do jj=1,i-1
               if (knum(jj) == 0) cycle
               do jk=1,knum(jj)
                  kns=kns+1
                  j=idpt(kns,1)
                  if (j == i) then
                     ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
                     xx=rx(jj)-rx(j)
                     yy=ry(jj)-ry(j)
                     zz=rz(jj)-rz(j)
                     call calc_replica(xx,yy,zz,rc_vor,ix,iy,iz,istat)
                     if (istat == 1) then
                        r2=xx**2+yy**2+zz**2
                        if (r2.lt.rc_vor2) then
                           nc=nc+1
                           xact(nc)=-xx; yact(nc)=-yy; zact(nc)=-zz
                        endif
                     endif
                  endif
               enddo
            enddo
         endif

!   neighboring atoms are remembered. now voronoi polyhedron search.
!   ('Computer Simulation' by Ueda, p.206)
         no=0
         nq=0
!         xj=rx(i); yj=ry(i); zj=rz(i)
         xj=0.0; yj=0.0; zj=0.0
         do k=1,nc-2
            x1=xact(k); y1=yact(k); z1=zact(k)
            a11=x1-xj; a12=y1-yj; a13=z1-zj
            do l=k+1,nc-1
               x2=xact(l); y2=yact(l); z2=zact(l)
               a21=x2-xj; a22=y2-yj; a23=z2-zj
               do m=l+1,nc
                  if ((k==i).or.(l==i).or.(m==i)) cycle
                  x3=xact(m); y3=yact(m); z3=zact(m)
                  a31=x3-xj; a32=y3-yj; a33=z3-zj
                  
                  b1=vor_f(x1-xj,y1-yj,z1-zj,x1+xj,y1+yj,z1+zj)/2.0d0
                  b2=vor_f(x2-xj,y2-yj,z2-zj,x2+xj,y2+yj,z2+zj)/2.0d0
                  b3=vor_f(x3-xj,y3-yj,z3-zj,x3+xj,y3+yj,z3+zj)/2.0d0
                  a1=a22*a33-a32*a23
                  a2=a32*a13-a12*a33
                  a3=a12*a23-a22*a13
                  dets=vor_f(a1,a2,a3,a11,a21,a31)
                  if (dets==0.0) cycle
                  c1=vor_f(a1,a2,a3,b1,b2,b3)/dets
                  a1=b3*a21-b2*a31
                  a2=b1*a31-b3*a11
                  a3=b2*a11-b1*a21
                  c2=vor_f(a1,a2,a3,a13,a23,a33)/dets
                  c3=-vor_f(a1,a2,a3,a12,a22,a32)/dets
                  a1=vor_e(c1-xj,c2-yj,c3-zj)

                  if (a1>rc_vor2) cycle
                  out=.false.
                  do ii=1,nc
                     if (ii==i) cycle
                     if (ii==k) cycle
                     if (ii==l) cycle
                     if (ii==m) cycle
                     a2=vor_e(xact(ii)-c1,yact(ii)-c2,zact(ii)-c3)
                     if (a2<=a1) then
                        out=.true.
                        exit
                     endif
                  enddo
                  if (out) cycle
                  no(k)=no(k)+1; no(l)=no(l)+1; no(m)=no(m)+1
                  nq=nq+1
                  np(nq,1)=k; np(nq,2)=l; np(nq,3)=m
                  rp(nq,1)=c1; rp(nq,2)=c2; rp(nq,3)=c3
               enddo
            enddo
         enddo
         if (nq>=1) then
!            print *,'i=',i,'nc=',nc,'nq=',nq
            na=0
            do ii=1,nc
               do ia=3,8
                  if (no(ii)==ia) na(ia)=na(ia)+1
               enddo
            enddo
            nat=0
            do ii=1,n
               nat=nat+na(ii)
            enddo
            do ifile=6,7
               write (ifile,'(i8," :",2x,i2," (",6i3," )")') i,nat,(na(ia),ia=3,8)
            enddo

! Polyhedra number count
            ntyp=0
! f=12
            if ((na(3)== 0).and.(na(4)== 0).and.(na(5)==12).and.(na(6)== 0)) ntyp=1
            if ((na(3)== 0).and.(na(4)== 2).and.(na(5)== 8).and.(na(6)== 2)) ntyp=2
            if ((na(3)== 0).and.(na(4)== 3).and.(na(5)== 6).and.(na(6)== 3)) ntyp=3
            if ((na(3)== 0).and.(na(4)== 4).and.(na(5)== 4).and.(na(6)== 4)) ntyp=4
! f=13
            if ((na(3)== 0).and.(na(4)== 1).and.(na(5)==10).and.(na(6)== 2)) ntyp=5
            if ((na(3)== 0).and.(na(4)== 2).and.(na(5)== 8).and.(na(6)== 3)) ntyp=6
            if ((na(3)== 0).and.(na(4)== 3).and.(na(5)== 6).and.(na(6)== 4)) ntyp=7
            if ((na(3)== 0).and.(na(4)== 4).and.(na(5)== 4).and.(na(6)== 5)) ntyp=8
            if ((na(3)== 0).and.(na(4)== 5).and.(na(5)== 2).and.(na(6)== 6)) ntyp=9
            if ((na(3)== 1).and.(na(4)== 0).and.(na(5)== 9).and.(na(6)== 3)) ntyp=10
! f=14
            if ((na(3)== 0).and.(na(4)== 1).and.(na(5)==10).and.(na(6)== 3)) ntyp=11
            if ((na(3)== 0).and.(na(4)== 2).and.(na(5)== 8).and.(na(6)== 4)) ntyp=12
            if ((na(3)== 0).and.(na(4)== 3).and.(na(5)== 6).and.(na(6)== 5)) ntyp=13
            if ((na(3)== 0).and.(na(4)== 4).and.(na(5)== 4).and.(na(6)== 6)) ntyp=14
! f=15
            if ((na(3)== 0).and.(na(4)== 1).and.(na(5)==10).and.(na(6)== 4)) ntyp=15
            if ((na(3)== 0).and.(na(4)== 2).and.(na(5)== 8).and.(na(6)== 5)) ntyp=16
            if ((na(3)== 0).and.(na(4)== 3).and.(na(5)== 6).and.(na(6)== 6)) ntyp=17
            if ((na(3)== 0).and.(na(4)== 4).and.(na(5)== 4).and.(na(6)== 7)) ntyp=18
            if ((na(3)== 0).and.(na(4)== 5).and.(na(5)== 2).and.(na(6)== 8)) ntyp=19
! f=16
            if ((na(3)== 0).and.(na(4)== 4).and.(na(5)== 4).and.(na(6)== 8)) ntyp=20
! others
            if ((na(3)== 0).and.(na(4)== 6).and.(na(5)== 0).and.(na(6)== 8)) ntyp=21
            if ((na(3)== 0).and.(na(4)== 0).and.(na(5)==12).and.(na(6)== 2)) ntyp=22

            if (ntyp/=0) nvor_count(ntyp)=nvor_count(ntyp)+1
               

!            print *,'nat=',nat
!            print *,no
!            print *,np
         endif
      enddo
      do ifile=6,7
      write (ifile,'("--COUNT RESULT--")')
      write (ifile,'("( 0, 0,12, 0):",i10,f8.2," %")') nvor_count( 1),real(nvor_count( 1))/real(n)*100.0
      write (ifile,'("( 0, 2, 8, 2):",i10,f8.2," %")') nvor_count( 2),real(nvor_count( 2))/real(n)*100.0
      write (ifile,'("( 0, 3, 6, 3):",i10,f8.2," %")') nvor_count( 3),real(nvor_count( 3))/real(n)*100.0
      write (ifile,'("( 0, 4, 4, 4):",i10,f8.2," %")') nvor_count( 4),real(nvor_count( 4))/real(n)*100.0

      write (ifile,'("( 0, 1,10, 2):",i10,f8.2," %")') nvor_count( 5),real(nvor_count( 5))/real(n)*100.0
      write (ifile,'("( 0, 2, 8, 3):",i10,f8.2," %")') nvor_count( 6),real(nvor_count( 6))/real(n)*100.0
      write (ifile,'("( 0, 3, 6, 4):",i10,f8.2," %")') nvor_count( 7),real(nvor_count( 7))/real(n)*100.0
      write (ifile,'("( 0, 4, 4, 5):",i10,f8.2," %")') nvor_count( 8),real(nvor_count( 8))/real(n)*100.0
      write (ifile,'("( 0, 5, 2, 6):",i10,f8.2," %")') nvor_count( 9),real(nvor_count( 9))/real(n)*100.0
      write (ifile,'("( 1, 0, 9, 3):",i10,f8.2," %")') nvor_count(10),real(nvor_count(10))/real(n)*100.0

      write (ifile,'("( 0, 1,10, 3):",i10,f8.2," %")') nvor_count(11),real(nvor_count(11))/real(n)*100.0
      write (ifile,'("( 0, 2, 8, 4):",i10,f8.2," %")') nvor_count(12),real(nvor_count(12))/real(n)*100.0
      write (ifile,'("( 0, 3, 6, 5):",i10,f8.2," %")') nvor_count(13),real(nvor_count(13))/real(n)*100.0
      write (ifile,'("( 0, 4, 4, 6):",i10,f8.2," %")') nvor_count(14),real(nvor_count(14))/real(n)*100.0

      write (ifile,'("( 0, 1,10, 4):",i10,f8.2," %")') nvor_count(15),real(nvor_count(15))/real(n)*100.0
      write (ifile,'("( 0, 2, 8, 5):",i10,f8.2," %")') nvor_count(16),real(nvor_count(16))/real(n)*100.0
      write (ifile,'("( 0, 3, 6, 6):",i10,f8.2," %")') nvor_count(17),real(nvor_count(17))/real(n)*100.0
      write (ifile,'("( 0, 4, 4, 7):",i10,f8.2," %")') nvor_count(18),real(nvor_count(18))/real(n)*100.0
      write (ifile,'("( 0, 5, 2, 8):",i10,f8.2," %")') nvor_count(19),real(nvor_count(19))/real(n)*100.0

      write (ifile,'("( 0, 4, 4, 8):",i10,f8.2," %")') nvor_count(20),real(nvor_count(20))/real(n)*100.0

      write (ifile,'("( 0, 6, 0, 8):",i10,f8.2," %")') nvor_count(21),real(nvor_count(21))/real(n)*100.0
      write (ifile,'("( 0, 0,12, 2):",i10,f8.2," %")') nvor_count(22),real(nvor_count(22))/real(n)*100.0
      enddo

      deallocate (xact,yact,zact)

      ifile=7
      close (ifile)

      end subroutine

      double precision function vor_f(x,y,z,a,b,c)
      implicit double precision (a-h,o-z)
      double precision :: x,y,z,a,b,c
      vor_f=a*x+b*y+c*z
      end function

      double precision function vor_e(a,b,c)
      implicit double precision (a-h,o-z)
      double precision :: a,b,c
      vor_e=a*a+b*b+c*c
      end function

